@@ -18,87 +18,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-/**
- * Flip frame vertically.
- * @param frame frame to flip
- */
-static void flip_frame_v(struct image_frame* frame)
-{
-    const size_t stride = frame->width * sizeof(argb_t);
-    void* buffer;
-
-    buffer = malloc(stride);
-    if (buffer) {
-        for (size_t y = 0; y < frame->height / 2; ++y) {
-            void* src = &frame->data[y * frame->width];
-            void* dst = &frame->data[(frame->height - y - 1) * frame->width];
-            memcpy(buffer, dst, stride);
-            memcpy(dst, src, stride);
-            memcpy(src, buffer, stride);
-        }
-        free(buffer);
-    }
-}
-
-/**
- * Flip frame horizontally.
- * @param frame frame to flip
- */
-static void flip_frame_h(struct image_frame* frame)
-{
-    for (size_t y = 0; y < frame->height; ++y) {
-        argb_t* line = &frame->data[y * frame->width];
-        for (size_t x = 0; x < frame->width / 2; ++x) {
-            argb_t* left = &line[x];
-            argb_t* right = &line[frame->width - x - 1];
-            const argb_t swap = *left;
-            *left = *right;
-            *right = swap;
-        }
-    }
-}
-
-/**
- * Rotate frame.
- * @param frame frame to rotate
- * @param angle rotation angle (only 90, 180, or 270)
- */
-static void rotate_frame(struct image_frame* frame, size_t angle)
-{
-    const size_t pixels = frame->width * frame->height;
-
-    if (angle == 180) {
-        for (size_t i = 0; i < pixels / 2; ++i) {
-            argb_t* color1 = &frame->data[i];
-            argb_t* color2 = &frame->data[pixels - i - 1];
-            const uint32_t swap = *color1;
-            *color1 = *color2;
-            *color2 = swap;
-        }
-    } else if (angle == 90 || angle == 270) {
-        const size_t buf_len = pixels * sizeof(argb_t);
-        argb_t* new_buffer = malloc(buf_len);
-        if (new_buffer) {
-            const size_t new_width = frame->height;
-            const size_t new_height = frame->width;
-            for (size_t y = 0; y < frame->height; ++y) {
-                for (size_t x = 0; x < frame->width; ++x) {
-                    size_t new_pos;
-                    if (angle == 90) {
-                        new_pos = x * new_width + (new_width - y - 1);
-                    } else {
-                        new_pos = (new_height - x - 1) * new_width + y;
-                    }
-                    new_buffer[new_pos] = frame->data[y * frame->width + x];
-                }
-            }
-            free(frame->data);
-            frame->width = new_width;
-            frame->height = new_height;
-            frame->data = new_buffer;
-        }
-    }
-}
 /**
  * Create image instance from memory buffer.
  * @param path path to the image
@@ -240,21 +159,21 @@ void image_free(struct image* ctx)
 void image_flip_vertical(struct image* ctx)
 {
     for (size_t i = 0; i < ctx->num_frames; ++i) {
-        flip_frame_v(&ctx->frames[i]);
+        pixmap_flip_vertical(&ctx->frames[i].pm);
     }
 }
 
 void image_flip_horizontal(struct image* ctx)
 {
     for (size_t i = 0; i < ctx->num_frames; ++i) {
-        flip_frame_h(&ctx->frames[i]);
+        pixmap_flip_horizontal(&ctx->frames[i].pm);
     }
 }
 
 void image_rotate(struct image* ctx, size_t angle)
 {
     for (size_t i = 0; i < ctx->num_frames; ++i) {
-        rotate_frame(&ctx->frames[i], angle);
+        pixmap_rotate(&ctx->frames[i].pm, angle);
     }
 }
 
@@ -294,7 +213,7 @@ void image_add_meta(struct image* ctx, const char* key, const char* fmt, ...)
     }
     ctx->info = buffer;
 
-    // contruct value string
+    // construct value string
     va_start(args, fmt);
     len = vsnprintf(NULL, 0, fmt, args);
     va_end(args);
@@ -315,12 +234,12 @@ void image_add_meta(struct image* ctx, const char* key, const char* fmt, ...)
     ++ctx->num_info;
 }
 
-struct image_frame* image_create_frame(struct image* ctx, size_t width,
-                                       size_t height)
+struct pixmap* image_allocate_frame(struct image* ctx, size_t width,
+                                    size_t height)
 {
     if (image_create_frames(ctx, 1) &&
-        image_frame_allocate(&ctx->frames[0], width, height)) {
-        return &ctx->frames[0];
+        pixmap_create(&ctx->frames[0].pm, width, height)) {
+        return &ctx->frames[0].pm;
     }
     image_free_frames(ctx);
     return NULL;
@@ -344,37 +263,9 @@ struct image_frame* image_create_frames(struct image* ctx, size_t num)
 void image_free_frames(struct image* ctx)
 {
     for (size_t i = 0; i < ctx->num_frames; ++i) {
-        free(ctx->frames[i].data);
+        pixmap_free(&ctx->frames[i].pm);
     }
     free(ctx->frames);
     ctx->frames = NULL;
     ctx->num_frames = 0;
 }
-
-bool image_frame_allocate(struct image_frame* frame, size_t width,
-                          size_t height)
-{
-    frame->data = malloc(width * height * sizeof(argb_t));
-    if (frame->data) {
-        frame->width = width;
-        frame->height = height;
-    } else {
-        image_print_error(NULL, "not enough memory");
-    }
-    return frame->data;
-}
-
-void image_print_error(const struct image* ctx, const char* fmt, ...)
-{
-    va_list args;
-
-    if (ctx) {
-        fprintf(stderr, "%s: ", ctx->file_name);
-    }
-
-    va_start(args, fmt);
-    vfprintf(stderr, fmt, args);
-    va_end(args);
-
-    fprintf(stderr, "\n");
-}
