@@ -14,6 +14,7 @@
 #include <errno.h>
 #include <limits.h>
 #include <pthread.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -43,7 +44,7 @@ struct image_list {
     size_t alloc;           ///< Number of allocated entries (size of array)
     size_t size;            ///< Number of files in array
     size_t index;           ///< Index of the current file entry
-    struct image* prev;     ///< Prevous image handle (cached)
+    struct image* prev;     ///< Previous image handle (cached)
     struct image* current;  ///< Current image handle
     struct image* next;     ///< Next image handle (preload)
     pthread_t preloader;    ///< Preload thread
@@ -60,7 +61,7 @@ static struct image_list ctx = {
     .order = order_alpha,
     .loop = true,
     .recursive = false,
-    .all_files = false,
+    .all_files = true,
 #ifdef HAVE_INOTIFY
     .notify = -1,
     .watch = -1,
@@ -379,24 +380,31 @@ static void watch_current(void)
 static void on_notify(void)
 {
     while (true) {
+        bool updated = false;
         uint8_t buffer[1024];
         ssize_t pos = 0;
         const ssize_t len = read(ctx.notify, buffer, sizeof(buffer));
+
         if (len < 0) {
             if (errno == EINTR) {
                 continue;
             }
             return; // something went wrong
         }
+
         while (pos + sizeof(struct inotify_event) <= (size_t)len) {
             const struct inotify_event* event =
                 (struct inotify_event*)&buffer[pos];
             if (event->mask & IN_IGNORED) {
                 ctx.watch = -1;
+            } else {
+                updated = true;
             }
             pos += sizeof(struct inotify_event) + event->len;
         }
-        viewer_reset();
+        if (updated) {
+            viewer_reset();
+        }
     }
 }
 #endif // HAVE_INOTIFY
@@ -408,21 +416,21 @@ static enum config_status load_config(const char* key, const char* value)
 {
     enum config_status status = cfgst_invalid_value;
 
-    if (strcmp(key, INFO_CFG_ORDER) == 0) {
+    if (strcmp(key, IMGLIST_CFG_ORDER) == 0) {
         const ssize_t index = str_index(order_names, value, 0);
         if (index >= 0) {
             ctx.order = index;
             status = cfgst_ok;
         }
-    } else if (strcmp(key, INFO_CFG_LOOP) == 0) {
+    } else if (strcmp(key, IMGLIST_CFG_LOOP) == 0) {
         if (config_to_bool(value, &ctx.loop)) {
             status = cfgst_ok;
         }
-    } else if (strcmp(key, INFO_CFG_RECURSIVE) == 0) {
+    } else if (strcmp(key, IMGLIST_CFG_RECURSIVE) == 0) {
         if (config_to_bool(value, &ctx.recursive)) {
             status = cfgst_ok;
         }
-    } else if (strcmp(key, INFO_CFG_ALL) == 0) {
+    } else if (strcmp(key, IMGLIST_CFG_ALL) == 0) {
         if (config_to_bool(value, &ctx.all_files)) {
             status = cfgst_ok;
         }
@@ -436,7 +444,7 @@ static enum config_status load_config(const char* key, const char* value)
 void image_list_init(void)
 {
     // register configuration loader
-    config_add_loader(INFO_CFG_SECTION, load_config);
+    config_add_loader(IMGLIST_CFG_SECTION, load_config);
 
 #ifdef HAVE_INOTIFY
     ctx.notify = inotify_init1(IN_CLOEXEC | IN_NONBLOCK);
