@@ -4,7 +4,7 @@
 
 #pragma once
 
-#include <stddef.h>
+#include "font.h"
 
 /** Info block position. */
 enum info_position {
@@ -17,10 +17,15 @@ enum info_position {
 
 /** Info line. */
 struct info_line {
-    wchar_t* key;
-    wchar_t* value;
+    struct text_surface key;
+    struct text_surface value;
 };
 
+/**
+ * Create info context.
+ */
+void info_create(void);
+
 /**
  * Initialize info context.
  */
@@ -51,7 +56,7 @@ void info_set_status(const char* fmt, ...)
     __attribute__((format(printf, 1, 2)));
 
 /**
- * Get number o f lines in the specified block.
+ * Get number of lines in the specified block.
  * @param pos block position
  * @return number of lines
  */
