@@ -1,11 +1,12 @@
 // SPDX-License-Identifier: MIT
-// User interface: Window managment, keyboard input, etc.
+// User interface: Window management, keyboard input, etc.
 // Copyright (C) 2020 Artem Senichev <artemsen@gmail.com>
 
 #include "ui.h"
 
 #include "buildcfg.h"
 #include "config.h"
+#include "keybind.h"
 #include "str.h"
 #include "viewer.h"
 #include "xdg-shell-protocol.h"
@@ -23,6 +24,11 @@
 // Max number of output displays
 #define MAX_OUTPUTS 4
 
+// Mouse button
+#ifndef BTN_LEFT
+#define BTN_LEFT 0x110 // from <linux/input-event-codes.h>
+#endif
+
 /** Loop state */
 enum state {
     state_ok,
@@ -76,7 +82,7 @@ struct ui {
         struct xdg_toplevel* toplevel;
     } xdg;
 
-    // keayboard
+    // keyboard
     struct xkb {
         struct xkb_context* context;
         struct xkb_keymap* keymap;
@@ -91,6 +97,13 @@ struct ui {
         uint32_t delay;
     } repeat;
 
+    // mouse drag
+    struct mouse {
+        bool active;
+        int x;
+        int y;
+    } mouse;
+
     // app_id name (window class)
     char* app_id;
 
@@ -281,7 +294,7 @@ static void on_keyboard_key(void* data, struct wl_keyboard* wl_keyboard,
         key += 8;
         keysym = xkb_state_key_get_one_sym(ctx.xkb.state, key);
         if (keysym != XKB_KEY_NoSymbol) {
-            viewer_on_keyboard(keysym);
+            viewer_on_keyboard(keysym, keybind_mods(ctx.xkb.state));
             // handle key repeat
             if (ctx.repeat.rate &&
                 xkb_keymap_key_repeats(ctx.xkb.keymap, key)) {
@@ -310,19 +323,40 @@ static void on_pointer_motion(void* data, struct wl_pointer* wl_pointer,
                               uint32_t time, wl_fixed_t surface_x,
                               wl_fixed_t surface_y)
 {
+    const int x = wl_fixed_to_int(surface_x);
+    const int y = wl_fixed_to_int(surface_y);
+
+    if (ctx.mouse.active) {
+        const int dx = x - ctx.mouse.x;
+        const int dy = y - ctx.mouse.y;
+        if (dx && dy) {
+            viewer_on_drag(dx, dy);
+        }
+    }
+
+    ctx.mouse.x = x;
+    ctx.mouse.y = y;
 }
 
 static void on_pointer_button(void* data, struct wl_pointer* wl_pointer,
                               uint32_t serial, uint32_t time, uint32_t button,
                               uint32_t state)
 {
+    if (button == BTN_LEFT) {
+        ctx.mouse.active = (state == WL_POINTER_BUTTON_STATE_PRESSED);
+    }
 }
 
 static void on_pointer_axis(void* data, struct wl_pointer* wl_pointer,
                             uint32_t time, uint32_t axis, wl_fixed_t value)
 {
-    xkb_keysym_t key = value > 0 ? XKB_KEY_SunPageDown : XKB_KEY_SunPageUp;
-    viewer_on_keyboard(key);
+    xkb_keysym_t key;
+    if (axis == WL_POINTER_AXIS_HORIZONTAL_SCROLL) {
+        key = value > 0 ? VKEY_SCROLL_RIGHT : VKEY_SCROLL_LEFT;
+    } else {
+        key = value > 0 ? VKEY_SCROLL_DOWN : VKEY_SCROLL_UP;
+    }
+    viewer_on_keyboard(key, keybind_mods(ctx.xkb.state));
 }
 
 static const struct wl_keyboard_listener keyboard_listener = {
@@ -651,20 +685,15 @@ static enum config_status load_config(const char* key, const char* value)
 void ui_init(void)
 {
     struct timespec ts;
-    char app_id[64];
-    size_t len;
 
     // create unique application id
     if (clock_gettime(CLOCK_MONOTONIC, &ts) == 0) {
+        char app_id[64];
         const uint64_t timestamp = (ts.tv_sec << 32) | ts.tv_nsec;
         snprintf(app_id, sizeof(app_id), APP_NAME "_%lx", timestamp);
+        str_dup(app_id, &ctx.app_id);
     } else {
-        strncpy(app_id, APP_NAME, sizeof(app_id));
-    }
-    len = strlen(app_id) + 1;
-    ctx.app_id = malloc(len);
-    if (ctx.app_id) {
-        memcpy(ctx.app_id, app_id, len);
+        str_dup(APP_NAME, &ctx.app_id);
     }
 
     // register configuration loader
@@ -783,8 +812,9 @@ bool ui_run(void)
             uint64_t repeats;
             const ssize_t sz = sizeof(repeats);
             if (read(ctx.repeat.fd, &repeats, sz) == sz) {
+                const uint8_t mods = keybind_mods(ctx.xkb.state);
                 while (repeats--) {
-                    viewer_on_keyboard(ctx.repeat.key);
+                    viewer_on_keyboard(ctx.repeat.key, mods);
                 }
             }
         }
@@ -809,10 +839,10 @@ void ui_stop(void)
 
 void ui_redraw(void)
 {
-    argb_t* wnd_data;
+    struct pixmap wnd = PIXMAP_ATTACH(NULL, ctx.wnd.width, ctx.wnd.height);
 
     if (!ctx.wnd.current) {
-        return; // not yet intialized
+        return; // not yet initialized
     }
 
     // switch buffers
@@ -823,8 +853,8 @@ void ui_redraw(void)
     }
 
     // draw to window buffer
-    wnd_data = wl_buffer_get_user_data(ctx.wnd.current);
-    viewer_on_redraw(wnd_data);
+    wnd.data = wl_buffer_get_user_data(ctx.wnd.current);
+    viewer_on_redraw(&wnd);
 
     // show window buffer
     wl_surface_attach(ctx.wl.surface, ctx.wnd.current, 0, 0);
