@@ -18,13 +18,15 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/timerfd.h>
+#include <sys/wait.h>
 #include <unistd.h>
 
 /** Viewer context. */
 struct viewer {
     size_t frame; ///< Index of the current frame
 
-    bool show_help; ///< Show/hide help message
+    struct text_surface* help; ///< Help lines
+    size_t help_sz;            ///< Number of lines in help
 
     bool animation_enable; ///< Animation enable/disable
     int animation_fd;      ///< Animation timer
@@ -40,6 +42,28 @@ static struct viewer ctx = { .animation_enable = true,
                              .slideshow_fd = -1,
                              .slideshow_time = 3 };
 
+static void switch_help(void)
+{
+    if (ctx.help) {
+        for (size_t i = 0; i < ctx.help_sz; i++) {
+            free(ctx.help[i].data);
+        }
+        free(ctx.help);
+        ctx.help = NULL;
+        ctx.help_sz = 0;
+    } else {
+        ctx.help_sz = 0;
+        ctx.help = calloc(1, key_bindings_size * sizeof(struct text_surface));
+        if (ctx.help) {
+            for (size_t i = 0; i < key_bindings_size; i++) {
+                if (key_bindings[i].help) {
+                    font_render(key_bindings[i].help, &ctx.help[ctx.help_sz++]);
+                }
+            }
+        }
+    }
+}
+
 /**
  * Switch to the next or previous frame.
  * @param forward switch direction
@@ -111,10 +135,10 @@ static void slideshow_ctl(bool enable)
 static void reset_state(void)
 {
     const struct image_entry entry = image_list_current();
-    const struct image_frame* frame = &entry.image->frames[0];
+    const struct pixmap* pm = &entry.image->frames[0].pm;
 
     ctx.frame = 0;
-    canvas_reset_image(frame->width, frame->height);
+    canvas_reset_image(pm->width, pm->height);
     ui_set_title(entry.image->file_name);
     animation_ctl(true);
     slideshow_ctl(ctx.slideshow_enable);
@@ -281,6 +305,9 @@ void viewer_init(void)
 
 void viewer_free(void)
 {
+    for (size_t i = 0; i < ctx.help_sz; i++) {
+        free(ctx.help[i].data);
+    }
     if (ctx.animation_fd != -1) {
         close(ctx.animation_fd);
     }
@@ -301,36 +328,26 @@ void viewer_reset(void)
     }
 }
 
-void viewer_on_redraw(argb_t* window)
+void viewer_on_redraw(struct pixmap* window)
 {
     const struct image_entry entry = image_list_current();
 
-    canvas_clear(window);
-    canvas_draw_image(entry.image->alpha, entry.image->frames[ctx.frame].data,
-                      window);
-
     info_update(ctx.frame);
+
+    canvas_draw_image(window, entry.image, ctx.frame);
+
+    // put text info blocks on window surface
     for (size_t i = 0; i < INFO_POSITION_NUM; ++i) {
         const size_t lines_num = info_height(i);
         if (lines_num) {
             const enum info_position pos = (enum info_position)i;
             const struct info_line* lines = info_lines(pos);
-            canvas_print(lines, lines_num, pos, window);
+            canvas_draw_text(window, pos, lines, lines_num);
         }
     }
 
-    if (ctx.show_help) {
-        const wchar_t** lines = malloc(key_bindings_size * sizeof(wchar_t*));
-        if (lines) {
-            size_t lines_num = 0;
-            for (size_t i = 0; i < key_bindings_size; ++i) {
-                if (key_bindings[i].help) {
-                    lines[lines_num++] = key_bindings[i].help;
-                }
-            }
-            canvas_print_center(lines, lines_num, window);
-            free(lines);
-        }
+    if (ctx.help) {
+        canvas_draw_ctext(window, ctx.help, ctx.help_sz);
     }
 
     // reset one-time rendered notification message
@@ -343,10 +360,10 @@ void viewer_on_resize(size_t width, size_t height, size_t scale)
     reset_state();
 }
 
-void viewer_on_keyboard(xkb_keysym_t key)
+void viewer_on_keyboard(xkb_keysym_t key, uint8_t mods)
 {
     bool redraw = false;
-    const struct key_binding* kbind = keybind_get(key);
+    const struct key_binding* kbind = keybind_get(key, mods);
 
     if (!kbind) {
         return;
@@ -357,7 +374,7 @@ void viewer_on_keyboard(xkb_keysym_t key)
         case kb_none:
             break;
         case kb_help:
-            ctx.show_help = !ctx.show_help;
+            switch_help();
             redraw = true;
             break;
         case kb_first_file:
@@ -444,7 +461,12 @@ void viewer_on_keyboard(xkb_keysym_t key)
             redraw = true;
             break;
         case kb_exit:
-            ui_stop();
+            if (ctx.help) {
+                switch_help(); // remove help overlay
+                redraw = true;
+            } else {
+                ui_stop();
+            }
             break;
     }
 
@@ -452,3 +474,10 @@ void viewer_on_keyboard(xkb_keysym_t key)
         ui_redraw();
     }
 }
+
+void viewer_on_drag(int dx, int dy)
+{
+    if (canvas_drag(dx, dy)) {
+        ui_redraw();
+    }
+}
