@@ -4,9 +4,8 @@
 
 #pragma once
 
-#include "types.h"
-
-#include <xkbcommon/xkbcommon.h>
+#include "keybind.h"
+#include "pixmap.h"
 
 // Configuration parameters
 #define VIEWER_CFG_SLIDESHOW      "slideshow"
@@ -29,9 +28,9 @@ void viewer_reset(void);
 
 /**
  * Redraw handler.
- * @param window pointer to window's pixel data
+ * @param window pixel map of window
  */
-void viewer_on_redraw(argb_t* window);
+void viewer_on_redraw(struct pixmap* window);
 
 /**
  * Window resize handler.
@@ -43,5 +42,12 @@ void viewer_on_resize(size_t width, size_t height, size_t scale);
 /**
  * Key press handler.
  * @param key code of key pressed
+ * @param mods key modifires (ctrl/alt/shift)
  */
-void viewer_on_keyboard(xkb_keysym_t key);
+void viewer_on_keyboard(xkb_keysym_t key, uint8_t mods);
+
+/**
+ * Image drap handler.
+ * @param dx,dy delta to move viewpoint
+ */
+void viewer_on_drag(int dx, int dy);
